<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Customer Group Catalog Hyva Compatibility by Amasty
 */

namespace Amasty\GroupcatHyva\Observer;

use Amasty\Groupcat\Model\ProductRuleProvider;
use Hyva\Theme\Service\CurrentTheme;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

/**
 * Due to the $associatedProducts of grouped product depends on the main product availability,
 * we need to disable hiding functionality of the main module for the
 * Magento_Catalog/templates/product/view/product-form.phtml
 */
class NotHideGroupedProduct implements ObserverInterface
{
    public const PRODUCT_TYPE_GROUPED = 'grouped';
    public const PRODUCT_FORM_FLAG = 'is_product_form_rendering';

    /**
     * @var CurrentTheme
     */
    private CurrentTheme $currentTheme;

    /**
     * @var ProductRuleProvider
     */
    private $ruleProvider;

    public function __construct(
        CurrentTheme $currentTheme,
        ProductRuleProvider $ruleProvider
    ) {
        $this->currentTheme = $currentTheme;
        $this->ruleProvider = $ruleProvider;
    }

    public function execute(Observer $observer)
    {
        if (!$this->currentTheme->isHyva()) {
            return;
        }

        $product = $observer->getData('item');

        if ($product->getTypeId() === self::PRODUCT_TYPE_GROUPED
            && $product->getData(self::PRODUCT_FORM_FLAG)
            && !$this->ruleProvider->getProductPriceAction($product)
        ) {
            $observer->setData('is_hide_addtocart', false);
            $product->setData(self::PRODUCT_FORM_FLAG, false);
        }
    }
}
