<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Customer Group Catalog Hyva Compatibility by Amasty
 */

namespace Amasty\GroupcatHyva\Plugin\Catalog\Block\Product\View;

use Amasty\Groupcat\Block\Framework\Pricing\HideAddTo;
use Amasty\Groupcat\Block\Framework\Pricing\RequestPopup;
use Amasty\Groupcat\Model\ProductRuleProvider;
use Amasty\Groupcat\Model\Rule\PriceActionOptionsProvider;
use Amasty\Groupcat\Model\Rule\Pricing\Render as PricingRuleRender;
use Amasty\GroupcatHyva\Observer\NotHideGroupedProduct;
use Hyva\Theme\Service\CurrentTheme;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\Pricing\Render as PricingRender;
use Magento\Framework\View\Element\Template;

class HidePrice
{
    public const PRODUCT_FORM_BLOCK_NAME = 'product.info.form';

    /**
     * @var PricingRuleRender
     */
    private PricingRuleRender $pricingRuleRender;

    /**
     * @var ProductRuleProvider
     */
    private ProductRuleProvider $productRuleProvider;

    /**
     * @var RequestPopup
     */
    private RequestPopup $requestPopup;

    /**
     * @var HideAddTo
     */
    private HideAddTo $hideAddTo;

    /**
     * @var CurrentTheme
     */
    private CurrentTheme $currentTheme;

    /**
     * @var string[]
     */
    private array $blocksToHide;

    /**
     * @var int[]
     */
    private array $renderedForProductIds = [];

    /**
     * @param PricingRuleRender $pricingRuleRender
     * @param ProductRuleProvider $productRuleProvider
     * @param RequestPopup $requestPopup
     * @param HideAddTo $hideAddTo
     * @param CurrentTheme $currentTheme
     * @param string[] $blocksToHide
     */
    public function __construct(
        PricingRuleRender $pricingRuleRender,
        ProductRuleProvider $productRuleProvider,
        RequestPopup $requestPopup,
        HideAddTo $hideAddTo,
        CurrentTheme $currentTheme,
        array $blocksToHide
    ) {
        $this->pricingRuleRender = $pricingRuleRender;
        $this->productRuleProvider = $productRuleProvider;
        $this->requestPopup = $requestPopup;
        $this->hideAddTo = $hideAddTo;
        $this->currentTheme = $currentTheme;
        $this->blocksToHide = $blocksToHide;
    }

    public function aroundToHtml(Template $subject, callable $proceed): string
    {
        if (!$this->currentTheme->isHyva()
            || !$subject->getProduct()
            || !in_array($subject->getNameInLayout(), $this->blocksToHide, true)
        ) {
            return $proceed();
        }

        if ($subject->getNameInLayout() === self::PRODUCT_FORM_BLOCK_NAME) {
            $subject->getProduct()->setData(NotHideGroupedProduct::PRODUCT_FORM_FLAG, true);
        }

        $result = $this->renderHideAddToButtons($subject->getProduct());

        if ($this->pricingRuleRender
            ->isNeedRenderPrice($subject->getProduct(), ['zone' => PricingRender::ZONE_ITEM_VIEW])) {
            $result .= $proceed();
        } else {
            $result .= $this->renderByRulePriceAction($subject->getProduct());
        }

        return $result;
    }

    public function renderByRulePriceAction(ProductInterface $product): string
    {
        $html = '';

        if (in_array($product->getId(), $this->renderedForProductIds, true)) {
            return $html;
        }

        switch ($this->productRuleProvider->getProductPriceAction($product)) {
            case PriceActionOptionsProvider::REPLACE:
                // replace price with CMS block
                $html = $this->pricingRuleRender->getPriceCmsBlockForZone($product, PricingRender::ZONE_ITEM_VIEW);
                $this->renderedForProductIds[] = $product->getId();
                break;
            case PriceActionOptionsProvider::REPLACE_REQUEST:
                // replace price with Request popup
                $html = $this->requestPopup->getProductRequestPrice($product);
                $this->renderedForProductIds[] = $product->getId();
                break;
        }

        return $html;
    }

    public function renderHideAddToButtons(ProductInterface $product): string
    {
        $html = '';

        if ($this->productRuleProvider->getProductIsHideCompare($product)
            || $this->productRuleProvider->getProductIsHideWishlist($product)
        ) {
            $html = $this->hideAddTo->getHideButtonsHtml($product);
        }

        return $html;
    }
}
